@@ -42,7 +42,6 @@ package com.jogamp.common.jvm;
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.security.AccessController;
@@ -52,8 +51,8 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 
+import com.jogamp.common.net.Uri;
 import com.jogamp.common.os.NativeLibrary;
-import com.jogamp.common.util.IOUtil;
 import com.jogamp.common.util.JarUtil;
 import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.common.util.cache.TempJarCache;
@@ -62,7 +61,18 @@ import jogamp.common.Debug;
 import jogamp.common.os.PlatformPropsImpl;
 
 public class JNILibLoaderBase {
-  public static final boolean DEBUG = Debug.debug("JNILibLoader");
+  public static final boolean DEBUG;
+  protected static final boolean PERF;
+
+  static {
+      Debug.initSingleton();
+      DEBUG = Debug.debug("JNILibLoader");
+      PERF = DEBUG || PropertyAccess.isPropertyDefined("jogamp.debug.JNILibLoader.Perf", true);
+  }
+
+  private static final Object perfSync = new Object();
+  private static long perfTotal = 0;
+  private static long perfCount = 0;
 
   public interface LoaderAction {
     /**
@@ -155,7 +165,7 @@ public class JNILibLoaderBase {
   /**
    *
    * @param classFromJavaJar
-   * @param classJarURI
+   * @param classJarUri
    * @param jarBasename jar basename w/ suffix
    * @param nativeJarBasename native jar basename w/ suffix
    * @return
@@ -163,69 +173,51 @@ public class JNILibLoaderBase {
    * @throws SecurityException
    * @throws URISyntaxException
    */
-  private static final boolean addNativeJarLibsImpl(final Class<?> classFromJavaJar, final URI classJarURI,
-                                                    final String jarBasename, final String nativeJarBasename)
+  private static final boolean addNativeJarLibsImpl(final Class<?> classFromJavaJar, final Uri classJarUri,
+                                                    final Uri.Encoded jarBasename, final Uri.Encoded nativeJarBasename)
     throws IOException, SecurityException, URISyntaxException
   {
     if (DEBUG) {
         final StringBuilder msg = new StringBuilder();
         msg.append("JNILibLoaderBase: addNativeJarLibsImpl(").append(PlatformPropsImpl.NEWLINE);
         msg.append("  classFromJavaJar  = ").append(classFromJavaJar).append(PlatformPropsImpl.NEWLINE);
-        msg.append("  classJarURI       = ").append(classJarURI).append(PlatformPropsImpl.NEWLINE);
+        msg.append("  classJarURI       = ").append(classJarUri).append(PlatformPropsImpl.NEWLINE);
         msg.append("  jarBasename       = ").append(jarBasename).append(PlatformPropsImpl.NEWLINE);
         msg.append("  os.and.arch       = ").append(PlatformPropsImpl.os_and_arch).append(PlatformPropsImpl.NEWLINE);
         msg.append("  nativeJarBasename = ").append(nativeJarBasename).append(PlatformPropsImpl.NEWLINE);
         msg.append(")");
         System.err.println(msg.toString());
     }
+    final long t0 = PERF ? System.currentTimeMillis() : 0; // 'Platform.currentTimeMillis()' not yet available!
 
     boolean ok = false;
 
-    final URI jarSubURI = JarUtil.getJarSubURI( classJarURI );
+    final Uri jarSubURI = classJarUri.getContainedUri();
     if (null == jarSubURI) {
-        throw new IllegalArgumentException("JarSubURI is null of: "+classJarURI);
+        throw new IllegalArgumentException("JarSubURI is null of: "+classJarUri);
     }
 
-    final String jarUriRoot_s = IOUtil.getURIDirname( jarSubURI.toString() );
+    final Uri jarSubUriRoot = jarSubURI.getDirectory();
 
     if (DEBUG) {
-        System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: initial: %s -> %s%n", jarSubURI, jarUriRoot_s);
+        System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: initial: %s -> %s%n", jarSubURI, jarSubUriRoot);
     }
 
     final String nativeLibraryPath = String.format("natives/%s/", PlatformPropsImpl.os_and_arch);
     if (DEBUG) {
         System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: nativeLibraryPath: %s%n", nativeLibraryPath);
     }
-    final ClassLoader cl = classFromJavaJar.getClassLoader();
-    final URL nativeLibraryURI = cl.getResource(nativeLibraryPath);
-    if (null != nativeLibraryURI) {
-        // We probably have one big-fat jar file, containing java classes
-        // and all native platform libraries under 'natives/os.and.arch'!
-        final URI nativeJarURI = JarUtil.getJarFileURI(jarUriRoot_s+jarBasename);
-        try {
-            if( TempJarCache.addNativeLibs(classFromJavaJar, nativeJarURI, nativeLibraryPath) ) {
-                ok = true;
-                if (DEBUG) {
-                    System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: fat: %s -> %s%n", jarBasename, nativeJarURI);
-                }
-            }
-        } catch(final Exception e) {
-            if(DEBUG) {
-                System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: Caught %s%n", e.getMessage());
-                e.printStackTrace();
-            }
-        }
-    }
-    if (!ok) {
-        // We assume one slim native jar file per 'os.and.arch'!
-        final URI nativeJarURI = JarUtil.getJarFileURI(jarUriRoot_s+nativeJarBasename);
+    {
+        // Attempt-1 a 'one slim native jar file' per 'os.and.arch' layout
+        // with native platform libraries under 'natives/os.and.arch'!
+        final Uri nativeJarURI = JarUtil.getJarFileUri( jarSubUriRoot.getEncoded().concat(nativeJarBasename) );
 
         if (DEBUG) {
             System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: module: %s -> %s%n", nativeJarBasename, nativeJarURI);
         }
 
         try {
-            ok = TempJarCache.addNativeLibs(classFromJavaJar, nativeJarURI, null /* nativeLibraryPath */);
+            ok = TempJarCache.addNativeLibs(classFromJavaJar, nativeJarURI, nativeLibraryPath);
         } catch(final Exception e) {
             if(DEBUG) {
                 System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: Caught %s%n", e.getMessage());
@@ -234,40 +226,75 @@ public class JNILibLoaderBase {
         }
     }
     if (!ok) {
-        // Attempt to find via ClassLoader and Native-Jar-Tag,
-        // assuming one slim native jar file per 'os.and.arch'!
-        final String moduleName;
+        final ClassLoader cl = classFromJavaJar.getClassLoader();
         {
-            final String packageName = classFromJavaJar.getPackage().getName();
-            final int idx = packageName.lastIndexOf('.');
-            if( 0 <= idx ) {
-                moduleName = packageName.substring(idx+1);
-            } else {
-                moduleName = packageName;
+            // Attempt-2 a 'one big-fat jar file' layout, containing java classes
+            // and all native platform libraries under 'natives/os.and.arch' per platform!
+            final URL nativeLibraryURI = cl.getResource(nativeLibraryPath);
+            if (null != nativeLibraryURI) {
+                final Uri nativeJarURI = JarUtil.getJarFileUri( jarSubUriRoot.getEncoded().concat(jarBasename) );
+                try {
+                    if( TempJarCache.addNativeLibs(classFromJavaJar, nativeJarURI, nativeLibraryPath) ) {
+                        ok = true;
+                        if (DEBUG) {
+                            System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: fat: %s -> %s%n", jarBasename, nativeJarURI);
+                        }
+                    }
+                } catch(final Exception e) {
+                    if(DEBUG) {
+                        System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: Caught %s%n", e.getMessage());
+                        e.printStackTrace();
+                    }
+                }
             }
         }
-        final String os_and_arch_dot = PlatformPropsImpl.os_and_arch.replace('-', '.');
-        final String nativeJarTagClassName = nativeJarTagPackage + "." + moduleName + "." + os_and_arch_dot + ".TAG" ; // TODO: sync with gluegen-cpptasks-base.xml
-        try {
-            if(DEBUG) {
-                System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: ClassLoader/TAG: Locating module %s, os.and.arch %s: %s%n",
-                        moduleName, os_and_arch_dot, nativeJarTagClassName);
+        if (!ok) {
+            // Attempt-3 to find via ClassLoader and Native-Jar-Tag,
+            // assuming one slim native jar file per 'os.and.arch'
+            // and native platform libraries under 'natives/os.and.arch'!
+            final String moduleName;
+            {
+                final String packageName = classFromJavaJar.getPackage().getName();
+                final int idx = packageName.lastIndexOf('.');
+                if( 0 <= idx ) {
+                    moduleName = packageName.substring(idx+1);
+                } else {
+                    moduleName = packageName;
+                }
             }
-            final URI nativeJarTagClassJarURI = JarUtil.getJarURI(nativeJarTagClassName, cl);
-            if (DEBUG) {
-                System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: ClassLoader/TAG: %s -> %s%n", nativeJarTagClassName, nativeJarTagClassJarURI);
-            }
-            ok = TempJarCache.addNativeLibs(classFromJavaJar, nativeJarTagClassJarURI, null /* nativeLibraryPath */);
-        } catch (final Exception e ) {
-            if(DEBUG) {
-                System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: Caught %s%n", e.getMessage());
-                e.printStackTrace();
+            final String os_and_arch_dot = PlatformPropsImpl.os_and_arch.replace('-', '.');
+            final String nativeJarTagClassName = nativeJarTagPackage + "." + moduleName + "." + os_and_arch_dot + ".TAG"; // TODO: sync with gluegen-cpptasks-base.xml
+            try {
+                if(DEBUG) {
+                    System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: ClassLoader/TAG: Locating module %s, os.and.arch %s: %s%n",
+                            moduleName, os_and_arch_dot, nativeJarTagClassName);
+                }
+                final Uri nativeJarTagClassJarURI = JarUtil.getJarUri(nativeJarTagClassName, cl);
+                if (DEBUG) {
+                    System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: ClassLoader/TAG: %s -> %s%n", nativeJarTagClassName, nativeJarTagClassJarURI);
+                }
+                ok = TempJarCache.addNativeLibs(classFromJavaJar, nativeJarTagClassJarURI, nativeLibraryPath);
+            } catch (final Exception e ) {
+                if(DEBUG) {
+                    System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: Caught %s%n", e.getMessage());
+                    e.printStackTrace();
+                }
             }
         }
     }
 
-    if (DEBUG) {
-        System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl: ok: %b%n", ok);
+    if (DEBUG || PERF) {
+        final long tNow = System.currentTimeMillis() - t0;
+        final long tTotal, tCount;
+        synchronized(perfSync) {
+            tCount = perfCount+1;
+            tTotal = perfTotal + tNow;
+            perfTotal = tTotal;
+            perfCount = tCount;
+        }
+        final double tAvrg = tTotal / (double)tCount;
+        System.err.printf("JNILibLoaderBase: addNativeJarLibsImpl.X: %s / %s -> ok: %b; duration: now %d ms, total %d ms (count %d, avrg %.3f ms)%n",
+                          jarBasename, nativeJarBasename, ok, tNow, tTotal, tCount, tAvrg);
     }
     return ok;
   }
@@ -399,14 +426,14 @@ public class JNILibLoaderBase {
               }
 
               final ClassLoader cl = c.getClassLoader();
-              final URI classJarURI = JarUtil.getJarURI(c.getName(), cl);
-              final String jarName = JarUtil.getJarBasename(classJarURI);
+              final Uri classJarURI = JarUtil.getJarUri(c.getName(), cl);
+              final Uri.Encoded jarName = JarUtil.getJarBasename(classJarURI);
 
               if (jarName == null) {
                   continue;
               }
 
-              final String jarBasename = jarName.substring(0, jarName.indexOf(".jar"));
+              final Uri.Encoded jarBasename = jarName.substring(0, jarName.indexOf(".jar"));
 
               if(DEBUG) {
                   System.err.printf("JNILibLoaderBase: jarBasename: %s%n", jarBasename);
@@ -423,7 +450,8 @@ public class JNILibLoaderBase {
                   }
               }
 
-              final String nativeJarBasename = String.format("%s-natives-%s.jar", jarBasename, PlatformPropsImpl.os_and_arch);
+              final Uri.Encoded nativeJarBasename =
+                      Uri.Encoded.cast( String.format("%s-natives-%s.jar", jarBasename.get(), PlatformPropsImpl.os_and_arch) );
 
               ok = JNILibLoaderBase.addNativeJarLibsImpl(c, classJarURI, jarName, nativeJarBasename);
               if (ok) {
@@ -585,7 +613,7 @@ public class JNILibLoaderBase {
                   if(DEBUG) {
                       System.err.println("ERROR (retry w/ enumLibPath) - "+ex1.getMessage());
                   }
-                  final List<String> possiblePaths = NativeLibrary.enumerateLibraryPaths(libraryName, libraryName, libraryName, true, cl);
+                  final List<String> possiblePaths = NativeLibrary.enumerateLibraryPaths(libraryName, libraryName, libraryName, cl);
                   // Iterate down these and see which one if any we can actually find.
                   for (final Iterator<String> iter = possiblePaths.iterator(); 0 == mode && iter.hasNext(); ) {
                       final String path = iter.next();
