@@ -50,12 +50,14 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.StringTokenizer;
 
-import jogamp.common.os.BionicDynamicLinkerImpl;
+import jogamp.common.os.BionicDynamicLinker32bitImpl;
+import jogamp.common.os.BionicDynamicLinker64BitImpl;
 import jogamp.common.os.MacOSXDynamicLinkerImpl;
 import jogamp.common.os.PlatformPropsImpl;
 import jogamp.common.os.PosixDynamicLinkerImpl;
 import jogamp.common.os.WindowsDynamicLinkerImpl;
 
+import com.jogamp.common.ExceptionUtils;
 import com.jogamp.common.util.IOUtil;
 import com.jogamp.common.util.cache.TempJarCache;
 
@@ -73,45 +75,42 @@ import com.jogamp.common.util.cache.TempJarCache;
     supporting code needed in the generated library. */
 
 public final class NativeLibrary implements DynamicLookupHelper {
-  private static final DynamicLinker dynLink;
   private static final String[] prefixes;
   private static final String[] suffixes;
+  private static final boolean isOSX;
 
   static {
     // Instantiate dynamic linker implementation
     switch (PlatformPropsImpl.OS_TYPE) {
       case WINDOWS:
-        dynLink = new WindowsDynamicLinkerImpl();
         prefixes = new String[] { "" };
         suffixes = new String[] { ".dll" };
+        isOSX = false;
         break;
 
       case MACOS:
-        dynLink = new MacOSXDynamicLinkerImpl();
         prefixes = new String[] { "lib" };
         suffixes = new String[] { ".dylib", ".jnilib" };
-        break;
-
-      case ANDROID:
-        dynLink = new BionicDynamicLinkerImpl();
-        prefixes = new String[] { "lib" };
-        suffixes = new String[] { ".so" };
+        isOSX = true;
         break;
 
       /*
+      case ANDROID:
       case FREEBSD:
       case SUNOS:
       case HPUX:
       case OPENKODE:
       case LINUX: */
       default:
-        dynLink = new PosixDynamicLinkerImpl();
         prefixes = new String[] { "lib" };
         suffixes = new String[] { ".so" };
+        isOSX = false;
         break;
     }
   }
 
+  private final DynamicLinker dynLink;
+
   // Platform-specific representation for the handle to the open
   // library. This is an HMODULE on Windows and a void* (the result of
   // a dlopen() call) on Unix and Mac OS X platforms.
@@ -123,7 +122,8 @@ public final class NativeLibrary implements DynamicLookupHelper {
   private final boolean global;
 
   // Private constructor to prevent arbitrary instances from floating around
-  private NativeLibrary(final long libraryHandle, final String libraryPath, final boolean global) {
+  private NativeLibrary(final DynamicLinker dynLink, final long libraryHandle, final String libraryPath, final boolean global) {
+    this.dynLink = dynLink;
     this.libraryHandle = libraryHandle;
     this.libraryPath   = libraryPath;
     this.global        = global;
@@ -134,22 +134,26 @@ public final class NativeLibrary implements DynamicLookupHelper {
 
   @Override
   public final String toString() {
-    return "NativeLibrary[" + libraryPath + ", 0x" + Long.toHexString(libraryHandle) + ", global " + global + "]";
+    return "NativeLibrary[" + dynLink.getClass().getSimpleName() + ", " + libraryPath + ", 0x" + Long.toHexString(libraryHandle) + ", global " + global + "]";
   }
 
   /** Opens the given native library, assuming it has the same base
       name on all platforms, looking first in the system's search
       path, and in the context of the specified ClassLoader, which is
-      used to help find the library in the case of e.g. Java Web Start. */
-  public static final NativeLibrary open(final String libName, final ClassLoader loader) {
+      used to help find the library in the case of e.g. Java Web Start.
+   * @throws SecurityException if user is not granted access for the named library.
+   */
+  public static final NativeLibrary open(final String libName, final ClassLoader loader) throws SecurityException {
     return open(libName, libName, libName, true, loader, true);
   }
 
   /** Opens the given native library, assuming it has the same base
       name on all platforms, looking first in the system's search
       path, and in the context of the specified ClassLoader, which is
-      used to help find the library in the case of e.g. Java Web Start. */
-  public static final NativeLibrary open(final String libName, final ClassLoader loader, final boolean global) {
+      used to help find the library in the case of e.g. Java Web Start.
+   * @throws SecurityException if user is not granted access for the named library.
+   */
+  public static final NativeLibrary open(final String libName, final ClassLoader loader, final boolean global) throws SecurityException {
     return open(libName, libName, libName, true, loader, global);
   }
 
@@ -157,9 +161,11 @@ public final class NativeLibrary implements DynamicLookupHelper {
       names (no "lib" prefix or ".dll/.so/.dylib" suffix) on the
       Windows, Unix and Mac OS X platforms, respectively, and in the
       context of the specified ClassLoader, which is used to help find
-      the library in the case of e.g. Java Web Start. The
-      searchSystemPathFirst argument changes the behavior to first
+      the library in the case of e.g. Java Web Start.
+      <p>
+      The {@code searchSystemPathFirst} argument changes the behavior to first
       search the default system path rather than searching it last.
+      </p>
       Note that we do not currently handle DSO versioning on Unix.
       Experience with JOAL and OpenAL has shown that it is extremely
       problematic to rely on a specific .so version (for one thing,
@@ -167,26 +173,55 @@ public final class NativeLibrary implements DynamicLookupHelper {
       ending in .so, for example .so.0), and in general if this
       dynamic loading facility is used correctly the version number
       will be irrelevant.
-  */
+   * @throws SecurityException if user is not granted access for the named library.
+   */
   public static final NativeLibrary open(final String windowsLibName,
                                          final String unixLibName,
                                          final String macOSXLibName,
                                          final boolean searchSystemPathFirst,
-                                         final ClassLoader loader) {
+                                         final ClassLoader loader) throws SecurityException {
     return open(windowsLibName, unixLibName, macOSXLibName, searchSystemPathFirst, loader, true);
   }
 
+  /**
+   * @throws SecurityException if user is not granted access for the named library.
+   */
   public static final NativeLibrary open(final String windowsLibName,
                                          final String unixLibName,
                                          final String macOSXLibName,
                                          final boolean searchSystemPathFirst,
-                                         final ClassLoader loader, final boolean global) {
+                                         final ClassLoader loader,
+                                         final boolean global) throws SecurityException {
     final List<String> possiblePaths = enumerateLibraryPaths(windowsLibName,
                                                        unixLibName,
                                                        macOSXLibName,
                                                        searchSystemPathFirst,
                                                        loader);
     Platform.initSingleton(); // loads native gluegen-rt library
+
+    final DynamicLinker dynLink;
+    switch (PlatformPropsImpl.OS_TYPE) {
+      case WINDOWS:
+        dynLink = new WindowsDynamicLinkerImpl();
+        break;
+
+      case MACOS:
+        dynLink = new MacOSXDynamicLinkerImpl();
+        break;
+
+      case ANDROID:
+        if( PlatformPropsImpl.CPU_ARCH.is32Bit ) {
+            dynLink = new BionicDynamicLinker32bitImpl();
+        } else {
+            dynLink = new BionicDynamicLinker64BitImpl();
+        }
+        break;
+
+      default:
+        dynLink = new PosixDynamicLinkerImpl();
+        break;
+    }
+
     // Iterate down these and see which one if any we can actually find.
     for (final Iterator<String> iter = possiblePaths.iterator(); iter.hasNext(); ) {
         final String path = iter.next();
@@ -206,7 +241,7 @@ public final class NativeLibrary implements DynamicLookupHelper {
             res = 0;
         }
         if ( 0 != res ) {
-            return new NativeLibrary(res, path, global);
+            return new NativeLibrary(dynLink, res, path, global);
         } else if( DEBUG ) {
             if( null != t ) {
                 System.err.println("NativeLibrary.open: Caught "+t.getClass().getSimpleName()+": "+t.getMessage());
@@ -233,7 +268,16 @@ public final class NativeLibrary implements DynamicLookupHelper {
   }
 
   @Override
-  public final long dynamicLookupFunction(final String funcName) {
+  public final void claimAllLinkPermission() throws SecurityException {
+      dynLink.claimAllLinkPermission();
+  }
+  @Override
+  public final void releaseAllLinkPermission() throws SecurityException {
+      dynLink.releaseAllLinkPermission();
+  }
+
+  @Override
+  public final long dynamicLookupFunction(final String funcName) throws SecurityException {
     if ( 0 == libraryHandle ) {
       throw new RuntimeException("Library is not open");
     }
@@ -241,22 +285,21 @@ public final class NativeLibrary implements DynamicLookupHelper {
   }
 
   @Override
-  public final boolean isFunctionAvailable(final String funcName) {
+  public final boolean isFunctionAvailable(final String funcName) throws SecurityException {
     if ( 0 == libraryHandle ) {
       throw new RuntimeException("Library is not open");
     }
     return 0 != dynLink.lookupSymbol(libraryHandle, funcName);
   }
 
-  /** Looks up the given function name in all loaded libraries. */
-  public static final long dynamicLookupFunctionGlobal(final String funcName) {
+  /** Looks up the given function name in all loaded libraries.
+   * @throws SecurityException if user is not granted access for the named library.
+   */
+  public final long dynamicLookupFunctionGlobal(final String funcName) throws SecurityException {
     return dynLink.lookupSymbolGlobal(funcName);
   }
 
-  /** Looks up the given function name in all loaded libraries. */
-  public static final boolean isFunctionAvailableGlobal(final String funcName) {
-    return 0 != dynLink.lookupSymbolGlobal(funcName);
-  }
+  /* pp */ final DynamicLinker getDynamicLinker() { return dynLink; }
 
   /** Retrieves the low-level library handle from this NativeLibrary
       object. On the Windows platform this is an HMODULE, and on Unix
@@ -271,8 +314,10 @@ public final class NativeLibrary implements DynamicLookupHelper {
   }
 
   /** Closes this native library. Further lookup operations are not
-      allowed after calling this method. */
-  public final void close() {
+      allowed after calling this method.
+   * @throws SecurityException if user is not granted access for the named library.
+   */
+  public final void close() throws SecurityException {
     if (DEBUG) {
       System.err.println("NativeLibrary.close(): closing " + this);
     }
@@ -281,10 +326,10 @@ public final class NativeLibrary implements DynamicLookupHelper {
     }
     final long handle = libraryHandle;
     libraryHandle = 0;
-    dynLink.closeLibrary(handle);
+    dynLink.closeLibrary(handle, DEBUG);
     if (DEBUG) {
       System.err.println("NativeLibrary.close(): Successfully closed " + this);
-      Thread.dumpStack();
+      ExceptionUtils.dumpStack(System.err);
     }
   }
 
@@ -325,10 +370,31 @@ public final class NativeLibrary implements DynamicLookupHelper {
 
   /** Given the base library names (no prefixes/suffixes) for the
       various platforms, enumerate the possible locations and names of
-      the indicated native library on the system. */
+      the indicated native library on the system not using the system path. */
   public static final List<String> enumerateLibraryPaths(final String windowsLibName,
                                                    final String unixLibName,
                                                    final String macOSXLibName,
+                                                   final ClassLoader loader) {
+      return enumerateLibraryPaths(windowsLibName, unixLibName, macOSXLibName,
+                                  false /* searchSystemPath */, false /* searchSystemPathFirst */,
+                                  loader);
+  }
+  /** Given the base library names (no prefixes/suffixes) for the
+      various platforms, enumerate the possible locations and names of
+      the indicated native library on the system using the system path. */
+  public static final List<String> enumerateLibraryPaths(final String windowsLibName,
+                                                   final String unixLibName,
+                                                   final String macOSXLibName,
+                                                   final boolean searchSystemPathFirst,
+                                                   final ClassLoader loader) {
+      return enumerateLibraryPaths(windowsLibName, unixLibName, macOSXLibName,
+                                  true /* searchSystemPath */, searchSystemPathFirst,
+                                  loader);
+  }
+  private static final List<String> enumerateLibraryPaths(final String windowsLibName,
+                                                   final String unixLibName,
+                                                   final String macOSXLibName,
+                                                   final boolean searchSystemPath,
                                                    final boolean searchSystemPathFirst,
                                                    final ClassLoader loader) {
     final List<String> paths = new ArrayList<String>();
@@ -346,11 +412,18 @@ public final class NativeLibrary implements DynamicLookupHelper {
 
     final String[] baseNames = buildNames(libName);
 
-    if (searchSystemPathFirst) {
-      // Add just the library names to use the OS's search algorithm
-      for (int i = 0; i < baseNames.length; i++) {
-        paths.add(baseNames[i]);
-      }
+    if( searchSystemPath && searchSystemPathFirst ) {
+        // Add just the library names to use the OS's search algorithm
+        for (int i = 0; i < baseNames.length; i++) {
+            paths.add(baseNames[i]);
+        }
+        // Add probable Mac OS X-specific paths
+        if ( isOSX ) {
+            // Add historical location
+            addPaths("/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
+            // Add current location
+            addPaths("/System/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
+        }
     }
 
     // The idea to ask the ClassLoader to find the library is borrowed
@@ -370,24 +443,25 @@ public final class NativeLibrary implements DynamicLookupHelper {
             if(null != usrPath) {
                 count++;
             }
-            final String sysPath = System.getProperty("sun.boot.library.path");
-            if(null != sysPath) {
-                count++;
+            final String sysPath;
+            if( searchSystemPath ) {
+                sysPath = System.getProperty("sun.boot.library.path");
+                if(null != sysPath) {
+                    count++;
+                }
+            } else {
+                sysPath = null;
             }
             final String[] res = new String[count];
             int i=0;
-            if (searchSystemPathFirst) {
-                if(null != sysPath) {
-                    res[i++] = sysPath;
-                }
+            if( null != sysPath && searchSystemPathFirst ) {
+                res[i++] = sysPath;
             }
             if(null != usrPath) {
                 res[i++] = usrPath;
             }
-            if (!searchSystemPathFirst) {
-                if(null != sysPath) {
-                    res[i++] = sysPath;
-                }
+            if( null != sysPath && !searchSystemPathFirst ) {
+                res[i++] = sysPath;
             }
             return res;
           }
@@ -411,19 +485,22 @@ public final class NativeLibrary implements DynamicLookupHelper {
         });
     addPaths(userDir, baseNames, paths);
 
-    if (!searchSystemPathFirst) {
-      // Add just the library names to use the OS's search algorithm
-      for (int i = 0; i < baseNames.length; i++) {
-        paths.add(baseNames[i]);
-      }
-    }
+    // Add current working directory + natives/os-arch/ + library names
+    // to handle Bug 1145 cc1 using an unpacked fat-jar
+    addPaths(userDir+File.separator+"natives"+File.separator+PlatformPropsImpl.os_and_arch+File.separator, baseNames, paths);
 
-    // Add probable Mac OS X-specific paths
-    if (PlatformPropsImpl.OS_TYPE == Platform.OSType.MACOS) {
-      // Add historical location
-      addPaths("/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
-      // Add current location
-      addPaths("/System/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
+    if( searchSystemPath && !searchSystemPathFirst ) {
+        // Add just the library names to use the OS's search algorithm
+        for (int i = 0; i < baseNames.length; i++) {
+            paths.add(baseNames[i]);
+        }
+        // Add probable Mac OS X-specific paths
+        if ( isOSX ) {
+            // Add historical location
+            addPaths("/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
+            // Add current location
+            addPaths("/System/Library/Frameworks/" + libName + ".Framework", baseNames, paths);
+        }
     }
 
     return paths;
@@ -440,13 +517,6 @@ public final class NativeLibrary implements DynamicLookupHelper {
       case MACOS:
         return macOSXLibName;
 
-      /*
-      case FREEBSD:
-      case DALVIK:
-      case SUNOS:
-      case HPUX:
-      case OPENKODE:
-      case LINUX: */
       default:
         return unixLibName;
     }
@@ -497,15 +567,14 @@ public final class NativeLibrary implements DynamicLookupHelper {
           }
       }
 
-      final String[] res = new String[prefixes.length * suffixes.length +
-                                ( PlatformPropsImpl.OS_TYPE == Platform.OSType.MACOS ? 1 : 0 )];
+      final String[] res = new String[prefixes.length * suffixes.length + ( isOSX ? 1 : 0 )];
       int idx = 0;
       for (int i = 0; i < prefixes.length; i++) {
           for (int j = 0; j < suffixes.length; j++) {
               res[idx++] = prefixes[i] + libName + suffixes[j];
           }
       }
-      if (PlatformPropsImpl.OS_TYPE == Platform.OSType.MACOS) {
+      if ( isOSX ) {
           // Plain library-base-name in Framework folder
           res[idx++] = libName;
       }
