@@ -28,12 +28,12 @@
 
 package com.jogamp.common.os;
 
-import java.net.URI;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.concurrent.TimeUnit;
 
 import com.jogamp.common.jvm.JNILibLoaderBase;
+import com.jogamp.common.net.Uri;
 import com.jogamp.common.util.JarUtil;
 import com.jogamp.common.util.PropertyAccess;
 import com.jogamp.common.util.ReflectionUtil;
@@ -41,7 +41,7 @@ import com.jogamp.common.util.VersionNumber;
 import com.jogamp.common.util.cache.TempJarCache;
 
 import jogamp.common.jvm.JVMUtil;
-import jogamp.common.os.MachineDescriptionRuntime;
+import jogamp.common.os.MachineDataInfoRuntime;
 import jogamp.common.os.PlatformPropsImpl;
 
 /**
@@ -50,7 +50,7 @@ import jogamp.common.os.PlatformPropsImpl;
  * Some field declarations and it's static initialization has been delegated
  * to it's super class {@link PlatformPropsImpl} to solve
  * static initialization interdependencies w/ the GlueGen native library loading
- * and it's derived information {@link #getMachineDescription()}, {@link #is32Bit()}, ..<br>
+ * and it's derived information {@link #getMachineDataInfo()}, {@link #is32Bit()}, ..<br>
  * This mechanism is preferred in this case to avoid synchronization and locking
  * and allow better performance accessing the mentioned fields/methods.
  * </p>
@@ -58,89 +58,199 @@ import jogamp.common.os.PlatformPropsImpl;
 public class Platform extends PlatformPropsImpl {
 
     public enum OSType {
-        LINUX(0), FREEBSD(1), ANDROID(2), MACOS(3), SUNOS(4), HPUX(5), WINDOWS(6), OPENKODE(7);
-
-        public final int id;
-
-        OSType(final int id){
-            this.id = id;
-        }
+        LINUX, FREEBSD, ANDROID, MACOS, SUNOS, HPUX, WINDOWS, OPENKODE;
     }
 
     public enum CPUFamily {
         /** AMD/Intel */
-        X86(    0x00000000),
+        X86,
         /** ARM */
-        ARM(    0x00010000),
+        ARM,
         /** Power PC */
-        PPC(    0x00020000),
+        PPC,
         /** SPARC */
-        SPARC(  0x00030000),
+        SPARC,
         /** Mips */
-        MIPS(   0x00040000),
+        MIPS,
         /** PA RISC */
-        PA_RISC(0xFFFF0000),
+        PA_RISC,
         /** Itanium */
-        IA64(   0xFFFF1000);
-
-        public final int id;
-
-        CPUFamily(final int id){
-            this.id = id;
-        }
+        IA64,
+        /** Hitachi SuperH */
+        SuperH;
     }
 
     public enum CPUType {
-        /** X86 32bit */
-        X86_32(    CPUFamily.X86,     0x0001),
-        /** X86 64bit */
-        X86_64(    CPUFamily.X86,     0x0002),
-        /** ARM default */
-        ARM(       CPUFamily.ARM,     0x0000),
-        /** ARM7EJ, ARM9E, ARM10E, XScale */
-        ARMv5(     CPUFamily.ARM,     0x0001),
-        /** ARM11 */
-        ARMv6(     CPUFamily.ARM,     0x0002),
-        /** ARM Cortex */
-        ARMv7(     CPUFamily.ARM,     0x0004),
-        /** PPC default */
-        PPC(       CPUFamily.PPC,     0x0000),
-        /** SPARC 32bit */
-        SPARC_32(  CPUFamily.SPARC,   0x0001),
-        /** SPARC 64bit */
-        SPARCV9_64(CPUFamily.SPARC,   0x0002),
-        /** MIPS 32bit */
-        MIPS_32(  CPUFamily.MIPS,     0x0001),
-        /** MIPS 64bit */
-        MIPS_64(  CPUFamily.MIPS,     0x0002),
-        /** Itanium default */
-        IA64(      CPUFamily.IA64,    0x0000),
-        /** PA_RISC2_0 */
-        PA_RISC2_0(CPUFamily.PA_RISC, 0x0001);
+        /** ARM 32bit default, usually little endian */
+        ARM(       CPUFamily.ARM,     true),
+        /** ARM7EJ, ARM9E, ARM10E, XScale, usually little endian */
+        ARMv5(     CPUFamily.ARM,     true),
+        /** ARM11, usually little endian */
+        ARMv6(     CPUFamily.ARM,     true),
+        /** ARM Cortex, usually little endian */
+        ARMv7(     CPUFamily.ARM,     true),
+        // 4
+
+        /** X86 32bit, little endian */
+        X86_32(    CPUFamily.X86,     true),
+        /** PPC 32bit default, usually big endian */
+        PPC(       CPUFamily.PPC,     true),
+        /** MIPS 32bit, big endian (mips) or little endian (mipsel) */
+        MIPS_32(   CPUFamily.MIPS,    true),
+        /** Hitachi SuperH 32bit default, ??? endian */
+        SuperH(    CPUFamily.SuperH,  true),
+        /** SPARC 32bit, big endian */
+        SPARC_32(  CPUFamily.SPARC,   true),
+        // 9
+
+        /** ARM64 default (64bit), usually little endian */
+        ARM64(     CPUFamily.ARM,     false),
+        /** ARM AArch64 (64bit), usually little endian */
+        ARMv8_A(   CPUFamily.ARM,     false),
+        /** X86 64bit, little endian */
+        X86_64(    CPUFamily.X86,     false),
+        /** PPC 64bit default, usually big endian */
+        PPC64(     CPUFamily.PPC,     false),
+        /** MIPS 64bit, big endian (mips64) or little endian (mipsel64) ? */
+        MIPS_64(   CPUFamily.MIPS,    false),
+        /** Itanium 64bit default, little endian */
+        IA64(      CPUFamily.IA64,    false),
+        /** SPARC 64bit, big endian */
+        SPARCV9_64(CPUFamily.SPARC,   false),
+        /** PA_RISC2_0 64bit, ??? endian */
+        PA_RISC2_0(CPUFamily.PA_RISC, false);
+        // 17
 
-        public final int id;
         public final CPUFamily family;
+        public final boolean is32Bit;
 
-        CPUType(final CPUFamily type, final int id){
+        CPUType(final CPUFamily type, final boolean is32Bit){
             this.family = type;
-            this.id = id;
+            this.is32Bit = is32Bit;
         }
 
-        public CPUFamily getFamily() { return family; }
+        /**
+         * Returns {@code true} if the given {@link CPUType} is compatible
+         * w/ this one, i.e. at least {@link #family} and {@link #is32Bit} is equal.
+         */
+        public final boolean isCompatible(final CPUType other) {
+            if( null == other ) {
+                return false;
+            } else if( other == this ) {
+                return true;
+            } else {
+                return this.family == other.family &&
+                       this.is32Bit == other.is32Bit;
+            }
+        }
+
+        public static final CPUType query(final String cpuABILower) {
+            if( null == cpuABILower ) {
+                throw new IllegalArgumentException("Null cpuABILower arg");
+            }
+            if(        cpuABILower.equals("x86")  ||
+                       cpuABILower.equals("i386") ||
+                       cpuABILower.equals("i486") ||
+                       cpuABILower.equals("i586") ||
+                       cpuABILower.equals("i686") ) {
+                return X86_32;
+            } else if( cpuABILower.equals("x86_64") ||
+                       cpuABILower.equals("amd64")  ) {
+                return X86_64;
+            } else if( cpuABILower.equals("ia64") ) {
+                return IA64;
+            } else if( cpuABILower.equals("aarch64") ) {
+                return ARM64;
+            } else if( cpuABILower.startsWith("arm") ) {
+                if(        cpuABILower.equals("armv8-a")   ||
+                           cpuABILower.equals("arm-v8-a") ||
+                           cpuABILower.equals("arm-8-a") ||
+                           cpuABILower.equals("arm64-v8a") ) {
+                    return ARMv8_A;
+                } else if( cpuABILower.startsWith("arm64") ) {
+                    return ARM64;
+                } else if( cpuABILower.startsWith("armv7") ||
+                           cpuABILower.startsWith("arm-v7") ||
+                           cpuABILower.startsWith("arm-7") ||
+                           cpuABILower.startsWith("armeabi-v7") ) {
+                    return ARMv7;
+                } else if( cpuABILower.startsWith("armv5") ||
+                           cpuABILower.startsWith("arm-v5") ||
+                           cpuABILower.startsWith("arm-5") ) {
+                    return ARMv5;
+                } else if( cpuABILower.startsWith("armv6") ||
+                           cpuABILower.startsWith("arm-v6") ||
+                           cpuABILower.startsWith("arm-6") ) {
+                    return ARMv6;
+                } else {
+                    return ARM;
+                }
+            } else if( cpuABILower.equals("sparcv9") ) {
+                return SPARCV9_64;
+            } else if( cpuABILower.equals("sparc") ) {
+                return SPARC_32;
+            } else if( cpuABILower.equals("pa_risc2.0") ) {
+                return PA_RISC2_0;
+            } else if( cpuABILower.startsWith("ppc64") ) {
+                return PPC64;
+            } else if( cpuABILower.startsWith("ppc") ) {
+                return PPC;
+            } else if( cpuABILower.startsWith("mips64") ) {
+                return MIPS_64;
+            } else if( cpuABILower.startsWith("mips") ) {
+                return MIPS_32;
+            } else if( cpuABILower.startsWith("superh") ) {
+                return SuperH;
+            } else {
+                throw new RuntimeException("Please port CPUType detection to your platform (CPU_ABI string '" + cpuABILower + "')");
+            }
+        }
     }
 
     public enum ABIType {
-        GENERIC_ABI    ( 0x0000 ),
+        GENERIC_ABI       ( 0x00 ),
         /** ARM GNU-EABI ARMEL -mfloat-abi=softfp */
-        EABI_GNU_ARMEL ( 0x0001 ),
+        EABI_GNU_ARMEL    ( 0x01 ),
         /** ARM GNU-EABI ARMHF -mfloat-abi=hard */
-        EABI_GNU_ARMHF ( 0x0002 );
+        EABI_GNU_ARMHF    ( 0x02 ),
+        /** ARM EABI AARCH64 (64bit) */
+        EABI_AARCH64      ( 0x03 );
 
         public final int id;
 
         ABIType(final int id){
             this.id = id;
         }
+
+        /**
+         * Returns {@code true} if the given {@link ABIType} is compatible
+         * w/ this one, i.e. they are equal.
+         */
+        public final boolean isCompatible(final ABIType other) {
+            if( null == other ) {
+                return false;
+            } else {
+                return other == this;
+            }
+        }
+
+        public static final ABIType query(final CPUType cpuType, final String cpuABILower) {
+            if( null == cpuType ) {
+                throw new IllegalArgumentException("Null cpuType");
+            } else if( null == cpuABILower ) {
+                throw new IllegalArgumentException("Null cpuABILower");
+            } else if( CPUFamily.ARM == cpuType.family ) {
+                if( !cpuType.is32Bit ) {
+                    return EABI_AARCH64;
+                } else if( cpuABILower.equals("armeabi-v7a-hard") ) {
+                    return EABI_GNU_ARMHF;
+                } else {
+                    return EABI_GNU_ARMEL;
+                }
+            } else {
+                return GENERIC_ABI;
+            }
+        }
     }
 
     private static final String useTempJarCachePropName = "jogamp.gluegen.UseTempJarCache";
@@ -162,9 +272,7 @@ public class Platform extends PlatformPropsImpl {
     // post loading native lib:
     //
 
-    private static final MachineDescription machineDescription;
-
-    private static final boolean is32Bit;
+    private static final MachineDataInfo machineDescription;
 
     /** <code>true</code> if AWT is available and not in headless mode, otherwise <code>false</code>. */
     public static final boolean AWT_AVAILABLE;
@@ -184,11 +292,11 @@ public class Platform extends PlatformPropsImpl {
 
                 final ClassLoader cl = Platform.class.getClassLoader();
 
-                final URI platformClassJarURI;
+                final Uri platformClassJarURI;
                 {
-                    URI _platformClassJarURI = null;
+                    Uri _platformClassJarURI = null;
                     try {
-                        _platformClassJarURI = JarUtil.getJarURI(Platform.class.getName(), cl);
+                        _platformClassJarURI = JarUtil.getJarUri(Platform.class.getName(), cl);
                     } catch (final Exception e) { }
                     platformClassJarURI = _platformClassJarURI;
                 }
@@ -225,21 +333,11 @@ public class Platform extends PlatformPropsImpl {
         USE_TEMP_JAR_CACHE = _USE_TEMP_JAR_CACHE[0];
         AWT_AVAILABLE = _AWT_AVAILABLE[0];
 
-        MachineDescription md = MachineDescriptionRuntime.getRuntime();
-        if(null == md) {
-            final MachineDescription.StaticConfig smd = MachineDescriptionRuntime.getStatic();
-            md = smd.md;
-            System.err.println("Warning: Using static MachineDescription: "+smd);
-        } else {
-            final MachineDescription.StaticConfig smd = MachineDescriptionRuntime.getStatic();
-            if(!md.compatible(smd.md)) {
-                throw new RuntimeException("Incompatible MachineDescriptions:"+PlatformPropsImpl.NEWLINE+
-                                           " Static "+smd+PlatformPropsImpl.NEWLINE+
-                                           " Runtime "+md);
-            }
-        }
-        machineDescription = md;
-        is32Bit = machineDescription.is32Bit();
+        //
+        // Validate and setup MachineDataInfo.StaticConfig
+        //
+        MachineDataInfoRuntime.initialize();
+        machineDescription = MachineDataInfoRuntime.getRuntime();
     }
 
     private Platform() {}
@@ -256,26 +354,6 @@ public class Platform extends PlatformPropsImpl {
      */
     public static void initSingleton() { }
 
-    /**
-     * Returns true only if having {@link java.nio.LongBuffer} and {@link java.nio.DoubleBuffer} available.
-     */
-    public static boolean isJavaSE() {
-        return JAVA_SE;
-    }
-
-    /**
-     * Returns true only if being compatible w/ language level 6, e.g. JRE 1.6.
-     * <p>
-     * Implies {@link #isJavaSE()}.
-     * </p>
-     * <p>
-     * <i>Note</i>: We claim Android is compatible.
-     * </p>
-     */
-    public static boolean isJava6() {
-        return JAVA_6;
-    }
-
     /**
      * Returns true if this machine is little endian, otherwise false.
      */
@@ -324,7 +402,7 @@ public class Platform extends PlatformPropsImpl {
      * Returns the CPU family.
      */
     public static CPUFamily getCPUFamily() {
-        return CPU_ARCH.getFamily();
+        return CPU_ARCH.family;
     }
 
     /**
@@ -334,6 +412,22 @@ public class Platform extends PlatformPropsImpl {
         return CPU_ARCH;
     }
 
+    /**
+     * Returns true if this JVM/ARCH is 32bit.
+     * <p>Shortcut to {@link #getCPUType()}.{@link CPUType#is32Bit is32Bit}</p>
+     */
+    public static boolean is32Bit() {
+        return CPU_ARCH.is32Bit; // used very often
+    }
+
+    /**
+     * Returns true if this JVM/ARCH is 64bit.
+     * <p>Shortcut to !{@link #getCPUType()}.{@link CPUType#is32Bit is32Bit}</p>
+     */
+    public static boolean is64Bit() {
+        return !CPU_ARCH.is32Bit; // used very often
+    }
+
     /**
      * Returns the ABI type.
      * <p>
@@ -407,27 +501,9 @@ public class Platform extends PlatformPropsImpl {
     }
 
     /**
-     * Returns true if this JVM/ARCH is 32bit.
-     * <p>Shortcut to {@link #getMachineDescription()}.{@link MachineDescription#is32Bit() is32Bit()}</p>
+     * Returns the MachineDataInfo of the running machine.
      */
-    public static boolean is32Bit() {
-        // return Platform.machineDescription.is32Bit();
-        return Platform.is32Bit; // used very often
-    }
-
-    /**
-     * Returns true if this JVM/ARCH is 64bit.
-     * <p>Shortcut to {@link #getMachineDescription()}.{@link MachineDescription#is32Bit() is64Bit()}</p>
-     */
-    public static boolean is64Bit() {
-        // return Platform.machineDescription.is64Bit();
-        return !Platform.is32Bit; // used very often
-    }
-
-    /**
-     * Returns the MachineDescription of the running machine.
-     */
-    public static MachineDescription getMachineDescription() {
+    public static MachineDataInfo getMachineDataInfo() {
         return machineDescription;
     }
 
