@@ -43,11 +43,13 @@ import com.jogamp.common.GlueGenVersion;
 
 import java.io.*;
 import java.util.*;
+import java.util.logging.Level;
 
 import antlr.*;
+
 import com.jogamp.gluegen.cgram.*;
 import com.jogamp.gluegen.cgram.types.*;
-import com.jogamp.gluegen.pcpp.*;
+import com.jogamp.gluegen.jcpp.JCPP;
 
 import static java.lang.System.*;
 
@@ -61,14 +63,18 @@ public class GlueGen implements GlueEmitterControls {
     }
 
     private final List<String> forcedStructNames = new ArrayList<String>();
-    private PCPP preprocessor;
+    private GenericCPP preprocessor;
 
     // State for SymbolFilters
-    private List<ConstantDefinition> constants;
-    private List<FunctionSymbol> functions;
+    private List<ConstantDefinition> allConstants;
+    private List<FunctionSymbol> allFunctions;
 
     private static boolean debug = false;
 
+    private static Level logLevel = null;
+
+    public static void setDebug(final boolean v) { debug=v; }
+    public static void setLogLevel(final Level l) { logLevel=l; }
     public static boolean debug() { return debug; }
 
     @Override
@@ -83,38 +89,66 @@ public class GlueGen implements GlueEmitterControls {
 
     @Override
     public void runSymbolFilter(final SymbolFilter filter) {
-        filter.filterSymbols(constants, functions);
+        filter.filterSymbols(allConstants, allFunctions);
         final List<ConstantDefinition> newConstants = filter.getConstants();
         final List<FunctionSymbol> newFunctions = filter.getFunctions();
         if (newConstants != null) {
-            constants = newConstants;
+            allConstants = newConstants;
         }
         if (newFunctions != null) {
-            functions = newFunctions;
+            allFunctions = newFunctions;
         }
     }
 
+    /** GlueGen's build in macro name {@value}, when compiling w/ GlueGen. */
+    public static final String __GLUEGEN__ = "__GLUEGEN__";
 
     @SuppressWarnings("unchecked")
-    public void run(final Reader reader, final String filename, final Class<?> emitterClass, final List<String> includePaths, final List<String> cfgFiles, final String outputRootDir, final boolean copyPCPPOutput2Stderr) {
+    public void run(final Reader reader, final String filename, final Class<?> emitterClass, final List<String> includePaths, final List<String> cfgFiles, final String outputRootDir, final boolean copyCPPOutput2Stderr) {
 
         try {
-            final File out = File.createTempFile("PCPPTemp", ".pcpp");
+            if(debug) {
+                Logging.getLogger().setLevel(Level.ALL);
+            } else if( null != logLevel ) {
+                Logging.getLogger().setLevel(logLevel);
+            }
+            final GlueEmitter emit;
+            if (emitterClass == null) {
+                emit = new JavaEmitter();
+            } else {
+                try {
+                    emit = (GlueEmitter) emitterClass.newInstance();
+                } catch (final Exception e) {
+                    throw new RuntimeException("Exception occurred while instantiating emitter class.", e);
+                }
+            }
+
+            for (final String config : cfgFiles) {
+                emit.readConfigurationFile(config);
+            }
+            final JavaConfiguration cfg = emit.getConfiguration();
+
+            final File out = File.createTempFile("CPPTemp", ".cpp");
             final FileOutputStream outStream = new FileOutputStream(out);
 
+            // preprocessor = new PCPP(includePaths, debug, copyCPPOutput2Stderr);
+            preprocessor = new JCPP(includePaths, debug, copyCPPOutput2Stderr);
+            final String cppName = preprocessor.getClass().getSimpleName();
             if(debug) {
-                System.err.println("PCPP output at (persistent): " + out.getAbsolutePath());
+                System.err.println("CPP <"+cppName+"> output at (persistent): " + out.getAbsolutePath());
             } else {
                 out.deleteOnExit();
             }
 
-            preprocessor = new PCPP(includePaths, debug, copyPCPPOutput2Stderr);
-            preprocessor.addDefine("__GLUEGEN__", "2");
+            preprocessor.addDefine(__GLUEGEN__, "2");
             preprocessor.setOut(outStream);
 
             preprocessor.run(reader, filename);
             outStream.flush();
             outStream.close();
+            if(debug) {
+                System.err.println("CPP <"+cppName+"> done");
+            }
 
             final FileInputStream inStream = new FileInputStream(out);
             final DataInputStream dis = new DataInputStream(inStream);
@@ -140,6 +174,7 @@ public class GlueGen implements GlueEmitterControls {
 
             final HeaderParser headerParser = new HeaderParser();
             headerParser.setDebug(debug);
+            headerParser.setJavaConfiguration(cfg);
             final TypeDictionary td = new TypeDictionary();
             headerParser.setTypedefDictionary(td);
             final TypeDictionary sd = new TypeDictionary();
@@ -162,21 +197,6 @@ public class GlueGen implements GlueEmitterControls {
             // generate glue code: the #defines to constants, the set of
             // typedefs, and the set of functions.
 
-            GlueEmitter emit = null;
-            if (emitterClass == null) {
-                emit = new JavaEmitter();
-            } else {
-                try {
-                    emit = (GlueEmitter) emitterClass.newInstance();
-                } catch (final Exception e) {
-                    throw new RuntimeException("Exception occurred while instantiating emitter class.", e);
-                }
-            }
-
-            for (final String config : cfgFiles) {
-                emit.readConfigurationFile(config);
-            }
-
             if (null != outputRootDir && outputRootDir.trim().length() > 0) {
                 if (emit instanceof JavaEmitter) {
                     // FIXME: hack to interfere with the *Configuration setting via commandlines
@@ -189,7 +209,7 @@ public class GlueGen implements GlueEmitterControls {
 
             // Repackage the enum and #define statements from the parser into a common format
             // so that SymbolFilters can operate upon both identically
-            constants = new ArrayList<ConstantDefinition>();
+            allConstants = new ArrayList<ConstantDefinition>();
             for (final EnumType enumeration : headerParser.getEnums()) {
                 String enumName = enumeration.getName();
                 if (enumName.equals("<anonymous>")) {
@@ -197,63 +217,93 @@ public class GlueGen implements GlueEmitterControls {
                 }
                 // iterate over all values in the enumeration
                 for (int i = 0; i < enumeration.getNumEnumerates(); ++i) {
-                    final String enumElementName = enumeration.getEnumName(i);
-                    final String value = String.valueOf(enumeration.getEnumValue(i));
-                    constants.add(new ConstantDefinition(enumElementName, value, true, enumName));
+                    final EnumType.Enumerator enumerate = enumeration.getEnum(i);
+                    final ConstantDefinition def =
+                            new ConstantDefinition(enumerate.getName(), enumerate.getExpr(),
+                                                   enumerate.getNumber(),
+                                                   enumName, enumeration.getASTLocusTag());
+                    allConstants.add(def);
                 }
             }
             for (final Object elem : lexer.getDefines()) {
                 final Define def = (Define) elem;
-                constants.add(new ConstantDefinition(def.getName(), def.getValue(), false, null));
+                allConstants.add(new ConstantDefinition(def.getName(), def.getValue(), null, def.getASTLocusTag()));
             }
+            allConstants.addAll(preprocessor.getConstantDefinitions());
 
-            functions = headerParser.getParsedFunctions();
+            allFunctions = headerParser.getParsedFunctions();
 
-            // begin emission of glue code
+            // begin emission of glue code,
+            // incl. firing up 'runSymbolFilter(SymbolFilter)' calls, which:
+            //    - filters all ConstantDefinition
+            //    - filters all FunctionSymbol
             emit.beginEmission(this);
 
-            emit.beginDefines();
-            final Set<String> emittedDefines = new HashSet<String>(100);
-            // emit java equivalent of enum { ... } statements
-            final StringBuilder comment = new StringBuilder();
-            for (final ConstantDefinition def : constants) {
-                if (!emittedDefines.contains(def.getName())) {
-                    emittedDefines.add(def.getName());
-                    final Set<String> aliases = def.getAliases();
-                    if (aliases != null) {
-                        comment.append("Alias for: <code>");
-                        for (final String alias : aliases) {
-                            comment.append(" ").append(alias);
-                        }
-                        comment.append("</code>");
-                    }
-                    if (def.getEnumName() != null) {
-                        if (comment.length() > 0)
-                            comment.append("<br>\n");
-
-                        comment.append("Defined as part of enum type \"");
-                        comment.append(def.getEnumName());
-                        comment.append("\"");
-                    }
-                    if (comment.length() > 0) {
-                        emit.emitDefine(def, comment.toString());
-                        comment.setLength(0);
-                    }
-                    else {
-                        emit.emitDefine(def, null);
+            if( debug() ) {
+                int i=0;
+                System.err.println("Filtered Constants: "+allConstants.size());
+                for (final ConstantDefinition def : allConstants) {
+                    if( debug() ) {
+                        System.err.println("Filtered ["+i+"]: "+def.getAliasedString());
+                        i++;
                     }
                 }
+                i=0;
+                System.err.println("Filtered Functions: "+allFunctions.size());
+                for (final FunctionSymbol cFunc : allFunctions) {
+                    System.err.println("Filtered ["+i+"]: "+cFunc.getAliasedString());
+                    i++;
+                }
+            }
+
+            if ( !cfg.structsOnly() ) {
+                emit.beginDefines();
+                final Set<String> emittedDefines = new HashSet<String>(100);
+                // emit java equivalent of enum { ... } statements
+                final StringBuilder comment = new StringBuilder();
+                for (final ConstantDefinition def : allConstants) {
+                    if (!emittedDefines.contains(def.getName())) {
+                        emittedDefines.add(def.getName());
+                        final Set<String> aliases = cfg.getAliasedDocNames(def);
+                        if (aliases != null && aliases.size() > 0 ) {
+                            int i=0;
+                            comment.append("Alias for: <code>");
+                            for (final String alias : aliases) {
+                                if(0 < i) {
+                                    comment.append("</code>, <code>");
+                                }
+                                comment.append(alias);
+                                i++;
+                            }
+                            comment.append("</code>");
+                        }
+                        if (def.getEnumName() != null) {
+                            if (comment.length() > 0)
+                                comment.append("<br>\n");
+
+                            comment.append("Defined as part of enum type \"");
+                            comment.append(def.getEnumName());
+                            comment.append("\"");
+                        }
+                        if (comment.length() > 0) {
+                            emit.emitDefine(def, comment.toString());
+                            comment.setLength(0);
+                        }
+                        else {
+                            emit.emitDefine(def, null);
+                        }
+                    }
+                }
+                emit.endDefines();
             }
-            emit.endDefines();
 
             // Iterate through the functions finding structs that are referenced in
             // the function signatures; these will be remembered for later emission
             final ReferencedStructs referencedStructs = new ReferencedStructs();
-            for (final FunctionSymbol sym : functions) {
+            for (final FunctionSymbol sym : allFunctions) {
                 // FIXME: this doesn't take into account the possibility that some of
                 // the functions we send to emitMethodBindings() might not actually be
-                // emitted (e.g., if an Ignore directive in the JavaEmitter causes it
-                // to be skipped).
+                // emitted (e.g., if an Ignore directive in the JavaEmitter causes it to be skipped).
                 sym.getType().visit(referencedStructs);
             }
 
@@ -273,13 +323,9 @@ public class GlueGen implements GlueEmitterControls {
 
             // Lay out structs
             emit.beginStructLayout();
-            for (final Iterator<Type> iter = referencedStructs.results(); iter.hasNext();) {
-                final Type t = iter.next();
-                if (t.isCompound()) {
-                    emit.layoutStruct(t.asCompound());
-                } else if (t.isPointer()) {
-                    final PointerType p = t.asPointer();
-                    final CompoundType c = p.getTargetType().asCompound();
+            for (final Iterator<CompoundType> iter = referencedStructs.layouts(); iter.hasNext();) {
+                final CompoundType c = iter.next();
+                if( !c.isLayouted() ) {
                     emit.layoutStruct(c);
                 }
             }
@@ -290,20 +336,23 @@ public class GlueGen implements GlueEmitterControls {
             for (final Iterator<Type> iter = referencedStructs.results(); iter.hasNext();) {
                 final Type t = iter.next();
                 if (t.isCompound()) {
+                    assert t.isTypedef() && t.getName() == null : "ReferencedStructs incorrectly recorded compound type " + t;
                     emit.emitStruct(t.asCompound(), null);
                 } else if (t.isPointer()) {
                     final PointerType p = t.asPointer();
                     final CompoundType c = p.getTargetType().asCompound();
-                    assert p.hasTypedefedName() && c.getName() == null : "ReferencedStructs incorrectly recorded pointer type " + p;
-                    emit.emitStruct(c, p.getName());
+                    assert p.isTypedef() && c.getName() == null : "ReferencedStructs incorrectly recorded pointer type " + p;
+                    emit.emitStruct(c, p);
                 }
             }
             emit.endStructs();
 
-            // emit java and C code to interface with the native functions
-            emit.beginFunctions(td, sd, headerParser.getCanonMap());
-            emit.emitFunctions(functions);
-            emit.endFunctions();
+            if ( !cfg.structsOnly() ) {
+                // emit java and C code to interface with the native functions
+                emit.beginFunctions(td, sd, headerParser.getCanonMap());
+                emit.emitFunctions(allFunctions);
+                emit.endFunctions();
+            }
 
             // end emission of glue code
             emit.endEmission();
@@ -340,6 +389,9 @@ public class GlueGen implements GlueEmitterControls {
                     emitterFQN = arg.substring(2);
                 } else if (arg.startsWith("-C")) {
                     cfgFiles.add(arg.substring(2));
+                } else if (arg.equals("--logLevel")) {
+                    i++;
+                    logLevel = Level.parse(args[i]);
                 } else if (arg.equals("--debug")) {
                     debug=true;
                 } else if (arg.equals("--dumpCPP")) {
@@ -392,7 +444,7 @@ public class GlueGen implements GlueEmitterControls {
         out.println("file or files can be specified with -C option; e.g,");
         out.println("-Cjava-emitter.cfg.");
         out.println("  --debug enables debug mode");
-        out.println("  --dumpCPP directs PCPP to dump all output to stderr as well");
+        out.println("  --dumpCPP directs CPP to dump all output to stderr as well");
         exit(1);
     }
 }
