@@ -56,16 +56,24 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.logging.Logger;
+import java.util.logging.Level;
+
+import com.jogamp.gluegen.ASTLocusTag;
+import com.jogamp.gluegen.ConstantDefinition;
+import com.jogamp.gluegen.GenericCPP;
+import com.jogamp.gluegen.GlueGenException;
+import com.jogamp.gluegen.Logging;
+import com.jogamp.gluegen.Logging.LoggerIf;
+
 import static java.util.logging.Level.*;
 
 /** A minimal pseudo-C-preprocessor designed in particular to preserve
     #define statements defining constants so they can be observed by a
     glue code generator. */
 
-public class PCPP {
+public class PCPP implements GenericCPP {
 
-    private static final Logger LOG = Logger.getLogger(PCPP.class.getPackage().getName());
+    private final LoggerIf LOG;
 
     /** Map containing the results of #define statements. We must
         evaluate certain very simple definitions (to properly handle
@@ -86,13 +94,15 @@ public class PCPP {
     private final boolean enableCopyOutput2Stderr;
 
     public PCPP(final List<String> includePaths, final boolean debug, final boolean copyOutput2Stderr) {
+        LOG = Logging.getLogger(PCPP.class.getPackage().getName(), PCPP.class.getSimpleName());
         this.includePaths = includePaths;
         setOut(System.out);
         enableDebugPrint = debug;
         enableCopyOutput2Stderr = copyOutput2Stderr;
     }
 
-    public void run(final Reader reader, final String filename) throws IOException {
+    @Override
+    public void run(final Reader reader, final String filename) throws GlueGenException {
         StreamTokenizer tok = null;
         BufferedReader bufReader = null;
         if (reader instanceof BufferedReader) {
@@ -108,13 +118,29 @@ public class PCPP {
         final ParseState oldState = state;
         state = curState;
         lineDirective();
-        parse();
+        try {
+            parse();
+        } catch (final Exception e) {
+            final StringBuilder buf = new StringBuilder("Preprocessor failed");
+            LOG.log(Level.SEVERE, buf.toString(), e);
+            if( e instanceof GlueGenException ) {
+                throw (GlueGenException)e;
+            } else {
+                throw new GlueGenException("Preprocessor failed",
+                                           new ASTLocusTag(filename(), lineNumber(), -1, null), e);
+            }
+        }
         state = oldState;
         if (state != null) {
             lineDirective();
         }
     }
 
+    @Override
+    public List<ConstantDefinition> getConstantDefinitions() throws GlueGenException {
+        return new ArrayList<ConstantDefinition>(); // NOP
+    }
+
     private void initTokenizer(final StreamTokenizer tok) {
         tok.resetSyntax();
         tok.wordChars('a', 'z');
@@ -131,6 +157,7 @@ public class PCPP {
         tok.slashStarComments(true);
     }
 
+    @Override
     public String findFile(final String filename) {
         final String sep = File.separator;
         for (final String inclPath : includePaths) {
@@ -143,10 +170,12 @@ public class PCPP {
         return null;
     }
 
+    @Override
     public OutputStream out() {
         return out;
     }
 
+    @Override
     public void setOut(final OutputStream out) {
         this.out = out;
         writer = new PrintWriter(out);
@@ -375,7 +404,7 @@ public class PCPP {
                             }
                         }
 
-                        if(isIdentifier(value)) {
+                        if(ConstantDefinition.isIdentifier(value)) {
                             newS +=" ";
                         }
 
@@ -459,28 +488,30 @@ public class PCPP {
         if (enabled()) {
             final String oldDef = defineMap.remove(name);
             if (oldDef == null) {
-                LOG.log(WARNING, "ignoring redundant \"#undef {0}\", at \"{1}\" line {2}: \"{3}\" was not previously defined",
-                        new Object[]{name, filename(), lineNumber(), name});
+                LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, name),
+                        "ignoring redundant \"#undef {0}\" - was not previously defined",
+                        name);
             } else {
                 // System.err.println("UNDEFINED: '" + name + "'  (line " + lineNumber() + " file " + filename() + ")");
             }
             nonConstantDefines.remove(name);
         } else {
-            LOG.log(WARNING, "FAILED TO UNDEFINE: ''{0}''  (line {1} file {2})", new Object[]{name, lineNumber(), filename()});
+            LOG.log(INFO, new ASTLocusTag(filename(), lineNumber(), -1, name),
+                    "DISABLED UNDEFINE: ''{0}''", name);
         }
     }
 
     private void handleWarning() throws IOException {
         final String msg = nextWordOrString();
         if (enabled()) {
-            LOG.log(WARNING, "#warning {0} at \"{1}\" line \"{2}\"", new Object[]{msg, filename(), lineNumber()});
+            LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, null), msg);
         }
     }
 
-    private void handleError() throws IOException {
+    private void handleError() throws IOException, GlueGenException {
         final String msg = nextWordOrString();
         if (enabled()) {
-            throw new RuntimeException("#error "+msg+" at \""+filename()+"\" line "+lineNumber());
+            throw new GlueGenException(msg, new ASTLocusTag(filename(), lineNumber(), -1, null));
         }
     }
 
@@ -520,6 +551,7 @@ public class PCPP {
         addDefine(name, macroDefinition, values);
     }
 
+    @Override
     public void addDefine(final String name, final String value) {
         final List<String> values = new ArrayList<String>();
         values.add(value);
@@ -541,7 +573,8 @@ public class PCPP {
                 final String value = "";
                 final String oldDef = defineMap.put(name, value);
                 if (oldDef != null && !oldDef.equals(value)) {
-                    LOG.log(WARNING, "\"{0}\" redefined from \"{1}\" to \"\"", new Object[]{name, oldDef});
+                    LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, null),
+                            "\"{0}\" redefined from \"{1}\" to \"\"", name, oldDef);
                 }
                 // We don't want to emit the define, because it would serve no purpose
                 // and cause GlueGen errors (confuse the GnuCParser)
@@ -551,12 +584,13 @@ public class PCPP {
                 // See whether the value is a constant
                 final String value = values.get(0);
 
-                if (isConstant(value)) {
+                if (ConstantDefinition.isNumber(value)) {
                     // Value is numeric constant like "#define FOO 5".
                     // Put it in the #define map
                     final String oldDef = defineMap.put(name, value);
                     if (oldDef != null && !oldDef.equals(value)) {
-                        LOG.log(WARNING, "\"{0}\" redefined from \"{1}\" to \"{2}\"", new Object[]{name, oldDef, value});
+                        LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, null),
+                                "\"{0}\" redefined from \"{1}\" to \"{2}\"", name, oldDef, value);
                     }
                     debugPrint(true, "DEFINE " + name + " ["+oldDef+" ] -> "+value + " CONST");
                     //System.err.println("//---DEFINED: " + name + " to \"" + value + "\"");
@@ -606,7 +640,8 @@ public class PCPP {
                 final Macro macro = new Macro(params, values);
                 final Macro oldDef = macroMap.put(name, macro);
                 if (oldDef != null) {
-                    LOG.log(WARNING, "\"{0}\" redefined from \"{1}\" to \"{2}\"", new Object[]{name, oldDef, macro});
+                    LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, null),
+                            "\"{0}\" redefined from \"{1}\" to \"{2}\"", name, oldDef, macro);
                 }
                 emitDefine = false;
 
@@ -618,7 +653,7 @@ public class PCPP {
 
                 boolean containsIdentifier = false;
                 for (final String value : values) {
-                    if(isIdentifier(value)) {
+                    if(ConstantDefinition.isIdentifier(value)) {
                         containsIdentifier = true;
                         break;
                     }
@@ -657,7 +692,8 @@ public class PCPP {
 
                     final String oldDef = defineMap.put(name, value);
                     if (oldDef != null && !oldDef.equals(value)) {
-                        LOG.log(WARNING, "\"{0}\" redefined from \"{1}\" to \"{2}\"", new Object[]{name, oldDef, value});
+                        LOG.log(WARNING, new ASTLocusTag(filename(), lineNumber(), -1, null),
+                                "\"{0}\" redefined from \"{1}\" to \"{2}\"", name, oldDef, value);
                     }
                     debugPrint(true, "DEFINE " + name + " ["+oldDef+" ] -> "+value + " CONST");
 //                    System.err.println("#define " + name +" "+value + " CONST EXPRESSION");
@@ -681,68 +717,6 @@ public class PCPP {
         //System.err.println("OUT HANDLE_DEFINE: " + name);
     }
 
-    private boolean isIdentifier(final String value) {
-
-        boolean identifier = false;
-
-        final char[] chars = value.toCharArray();
-
-        for (int i = 0; i < chars.length; i++) {
-            final char c = chars[i];
-            if (i == 0) {
-                if (Character.isJavaIdentifierStart(c)) {
-                    identifier = true;
-                }
-            } else {
-                if (!Character.isJavaIdentifierPart(c)) {
-                    identifier = false;
-                    break;
-                }
-            }
-        }
-        return identifier;
-    }
-
-    private boolean isConstant(final String s) {
-        if (s.startsWith("0x") || s.startsWith("0X")) {
-            return checkHex(s);
-        } else {
-            return checkDecimal(s);
-        }
-    }
-
-    private boolean checkHex(final String s) {
-        char c='\0';
-        int i;
-        for (i = 2; i < s.length(); i++) {
-            c = s.charAt(i);
-            if (!((c >= '0' && c <= '9') ||
-                  (c >= 'a' && c <= 'f') ||
-                  (c >= 'A' && c <= 'F'))) {
-                break;
-            }
-        }
-        if(i==s.length()) {
-            return true;
-        } else if(i==s.length()-1) {
-            // Const qualifier ..
-            return c == 'l' || c == 'L' ||
-                   c == 'f' || c == 'F' ||
-                   c == 'u' || c == 'U' ;
-        }
-        return false;
-    }
-
-    private boolean checkDecimal(final String s) {
-        try {
-            Float.valueOf(s);
-        } catch (final NumberFormatException e) {
-            // not parsable as a number
-            return false;
-        }
-        return true;
-    }
-
     private String resolveDefine(final String word, final boolean returnNullIfNotFound) {
         String lastWord = defineMap.get(word);
         if (lastWord == null) {
@@ -920,6 +894,27 @@ public class PCPP {
                         ifValue = false;
                     }
                     break;
+                case '*':
+                    {
+                        // NOTE: we don't handle expressions like this properly
+                        final boolean rhs = handleIfRecursive(false);
+                        ifValue = false;
+                    }
+                    break;
+                case '+':
+                    {
+                        // NOTE: we don't handle expressions like this properly
+                        final boolean rhs = handleIfRecursive(false);
+                        ifValue = false;
+                    }
+                    break;
+                case '-':
+                    {
+                        // NOTE: we don't handle expressions like this properly
+                        final boolean rhs = handleIfRecursive(false);
+                        ifValue = false;
+                    }
+                    break;
                 case '=':
                     {
                         // NOTE: we don't handle expressions like this properly
@@ -1008,7 +1003,8 @@ public class PCPP {
                 buf.append(curTokenAsString());
             }
             if (t == StreamTokenizer.TT_EOF) {
-                LOG.warning("unexpected EOF while processing #include directive");
+                LOG.warning(new ASTLocusTag(filename(), lineNumber(), -1, null),
+                            "unexpected EOF while processing #include directive");
             }
             filename = buf.toString();
         }
