@@ -30,9 +30,9 @@ package com.jogamp.gluegen.test.junit.generation;
 
 import com.jogamp.common.nio.Buffers;
 import com.jogamp.common.nio.PointerBuffer;
-import com.jogamp.common.os.MachineDescription;
+import com.jogamp.common.os.MachineDataInfo;
 import com.jogamp.common.os.Platform;
-import com.jogamp.junit.util.JunitTracer;
+import com.jogamp.junit.util.SingletonJunitCase;
 
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
@@ -41,7 +41,7 @@ import java.nio.IntBuffer;
 import java.nio.LongBuffer;
 import java.util.Arrays;
 
-import jogamp.common.os.MachineDescriptionRuntime;
+import jogamp.common.os.MachineDataInfoRuntime;
 
 import org.junit.Assert;
 
@@ -50,7 +50,7 @@ import org.junit.Assert;
  * @author Michael Bien
  * @author Sven Gothel
  */
-public class BaseClass extends JunitTracer {
+public class BaseClass extends SingletonJunitCase {
 
     /**
      * Verifies the existence and creation of the generated class.
@@ -65,21 +65,25 @@ public class BaseClass extends JunitTracer {
         Assert.assertNotNull(ifName+" does not exist", clazzIf);
         Assert.assertNotNull(implName+" does not exist", clazzImpl);
 
-        Assert.assertEquals(1,  clazzIf.getDeclaredField("CONSTANT_ONE").get(null));
+        Assert.assertNotNull(clazzImpl.getDeclaredMethod("nopTest"));
 
         final Object obj = clazzImpl.newInstance();
         Assert.assertTrue("Not of type "+ifName, clazzIf.isAssignableFrom(obj.getClass()));
-        Assert.assertTrue("Not of type com.jogamp.gluegen.test.junit.generation.Bindingtest1", (obj instanceof com.jogamp.gluegen.test.junit.generation.Bindingtest1));
+        Assert.assertTrue("Not of type com.jogamp.gluegen.test.junit.generation.Bindingtest1",
+                (obj instanceof com.jogamp.gluegen.test.junit.generation.Bindingtest1));
     }
 
+    public static final float EPSILON = 1.1920929E-7f; // Float.MIN_VALUE == 1.4e-45f ; double EPSILON 2.220446049250313E-16d
+
     /**
      * Verifies if all generated method signatures are completed,
      * ie a compilation only coverage test without functional tests.
      */
     public void chapter__TestCoverageSignature(final Bindingtest1 binding) throws Exception {
-          int i;
+          int i = 0;
           final long context = 0;
           LongBuffer lb=null;
+          ByteBuffer bb=null;
           final IntBuffer ib=null;
           final long[] larray = null;
           final int larray_offset = 0;
@@ -89,10 +93,42 @@ public class BaseClass extends JunitTracer {
           final int iarray_offset = 0;
           long result = 0;
           long l = result;
+          ShortBlob sb = null;
+          Int32Struct i32s = null;
+          AnonBlob ab = null;
+          PointerBuffer pb=null;
 
           {
-              final ByteBuffer bb = binding.createAPtrBlob();
-              PointerBuffer pb = safeByteBuffer2PointerBuffer(bb, 1);
+              l = binding.testXID(l);
+              l = binding.testXID_2(l);
+
+              bb = binding.testAnonBuffer(bb);
+
+              sb = binding.testShortBlob(sb);
+              sb = binding.testLPShortBlob0(sb);
+              sb = binding.testLPShortBlob1(sb);
+              sb = binding.testLPShortBlob2(sb);
+              sb = binding.testLPShortBlob3(sb);
+              sb = binding.testShortBlobL1(sb);
+              sb = binding.testShortBlobL2(sb);
+
+              i32s = binding.testInt32Struct(i32s);
+
+              ab = binding.testCreateAnonBlob();
+              binding.testDestroyAnonBlob(ab);
+
+              l = binding.testCreateAnonBlob2();
+              binding.testDestroyAnonBlob2(l);
+
+              lb = binding.testFooPtr(larray, 0);
+              lb = binding.testFooPtr(lb);
+
+              i = binding.testDelegate(i);
+          }
+
+          {
+              bb = binding.createAPtrBlob();
+              pb = safeByteBuffer2PointerBuffer(bb, 1);
               long bb2A = binding.getAPtrAddress(bb);
               bb2A = bb2A - 0; // avoid warning
 
@@ -119,9 +155,6 @@ public class BaseClass extends JunitTracer {
               binding.releaseAPtrBlob(bb);
           }
 
-          final ByteBuffer bb=null;
-          PointerBuffer pb=null;
-
           result = binding.arrayTestInt32(context, ib);
           result = binding.arrayTestInt32(context, iarray, iarray_offset);
 
@@ -177,6 +210,185 @@ public class BaseClass extends JunitTracer {
           l = binding.typeTestUIntPtrT(l, l);
     }
 
+    /**
+     * Verifies if all generated static constant values are completed,
+     * and whether their value is as expected!
+     * <p>
+     * Covers all enumerates and defines.
+     * </p>
+     */
+    public void chapter01TestStaticConstants(final Bindingtest1 binding) throws Exception {
+        // Plain vanilla CPP constants
+        Assert.assertEquals(   1, Bindingtest1.CONSTANT_ONE);
+        Assert.assertEquals(   8, Bindingtest1.ARRAY_SIZE);
+        Assert.assertEquals(1234, Bindingtest1.DEFINE_01);
+
+        // Enums
+        Assert.assertEquals(   1, Bindingtest1.LI);
+        Assert.assertEquals(   3, Bindingtest1.LO);
+        Assert.assertEquals(   2, Bindingtest1.LU);
+        Assert.assertEquals(   1, Bindingtest1.MI);
+        Assert.assertEquals(   3, Bindingtest1.MO);
+        Assert.assertEquals(   2, Bindingtest1.MU);
+        Assert.assertEquals(   0, Bindingtest1.ZERO);
+        Assert.assertEquals(   1, Bindingtest1.ONE);
+        Assert.assertEquals(   2, Bindingtest1.TWO);
+        Assert.assertEquals(   3, Bindingtest1.THREE);
+
+        // CPP Macro Expansion!
+        Assert.assertEquals(   1, Bindingtest1.NUMBER_ONE);
+        Assert.assertEquals(   2, Bindingtest1.NUMBER_TWO);
+        Assert.assertEquals(   4, Bindingtest1.NUMBER_FOUR);
+        Assert.assertEquals(   5, Bindingtest1.NUMBER_FIVE);
+        Assert.assertEquals(   8, Bindingtest1.NUMBER_EIGHT);
+        Assert.assertEquals(   9, Bindingtest1.NUMBER_NINE);
+        Assert.assertEquals(  10, Bindingtest1.NUMBER_TEN);
+
+        // Enum Constant Expressions!
+        Assert.assertEquals(   1, Bindingtest1.ENUM_NUM_ONE);
+        Assert.assertEquals(   2, Bindingtest1.ENUM_NUM_TWO);
+        Assert.assertEquals(   3, Bindingtest1.ENUM_NUM_THREE);
+        Assert.assertEquals(   4, Bindingtest1.ENUM_NUM_FOUR);
+        Assert.assertEquals(   5, Bindingtest1.ENUM_NUM_FIVE);
+        Assert.assertEquals(   8, Bindingtest1.ENUM_NUM_EIGHT);
+        Assert.assertEquals(   9, Bindingtest1.ENUM_NUM_NINE);
+        Assert.assertEquals(  10, Bindingtest1.ENUM_NUM_TEN);
+
+        // Integer 32bit (int / enum)
+        final int ENUM_I0 =  Bindingtest1.ENUM_I0;
+        final int ENUM_I1 =  Bindingtest1.ENUM_I1;
+        final int ENUM_I2 =  Bindingtest1.ENUM_I2;
+        final int ENUM_I3 =  Bindingtest1.ENUM_I3;
+        final int ENUM_I4 = -Bindingtest1.ENUM_I4;
+        final int ENUM_I5 = -Bindingtest1.ENUM_I5;
+        final int ENUM_I6 = -Bindingtest1.ENUM_I6;
+        final int ENUM_I7 =  Bindingtest1.ENUM_I7;
+        final int ENUM_I8 =  Bindingtest1.ENUM_I8;
+        final int ENUM_I9 =  Bindingtest1.ENUM_I9;
+        final int ENUM_IA =  Bindingtest1.ENUM_IA;
+        final int ENUM_IB =  Bindingtest1.ENUM_IB;
+        final int ENUM_IX =  Bindingtest1.ENUM_IX;
+        int iexp = 10;
+        Assert.assertEquals(iexp++, ENUM_I0);
+        Assert.assertEquals(iexp++, ENUM_I1);
+        Assert.assertEquals(iexp++, ENUM_I2);
+        Assert.assertEquals(iexp++, ENUM_I3);
+        Assert.assertEquals(iexp++, ENUM_I4);
+        Assert.assertEquals(iexp++, ENUM_I5);
+        Assert.assertEquals(iexp++, ENUM_I6);
+        Assert.assertEquals(iexp++, ENUM_I7);
+        Assert.assertEquals(iexp++, ENUM_I8);
+        Assert.assertEquals(iexp++, ENUM_I9);
+        Assert.assertEquals(iexp++, ENUM_IA);
+        Assert.assertEquals(iexp++, ENUM_IB);
+        Assert.assertEquals(0xffffffff, ENUM_IX);
+
+        // Integer 32bit (int / define)
+        final int CL_INT_I0 =  Bindingtest1.CL_INT_I0;
+        final int CL_INT_I1 =  Bindingtest1.CL_INT_I1;
+        final int CL_INT_I2 =  Bindingtest1.CL_INT_I2;
+        final int CL_INT_I3 =  Bindingtest1.CL_INT_I3;
+        final int CL_INT_I4 = -Bindingtest1.CL_INT_I4;
+        final int CL_INT_I5 = -Bindingtest1.CL_INT_I5;
+        final int CL_INT_I6 = -Bindingtest1.CL_INT_I6;
+        final int CL_INT_I7 = -Bindingtest1.CL_INT_I7;
+        final int CL_INT_I8 =  Bindingtest1.CL_INT_I8;
+        final int CL_INT_I9 =  Bindingtest1.CL_INT_I9;
+        final int CL_INT_IA =  Bindingtest1.CL_INT_IA;
+        final int CL_INT_IB =  Bindingtest1.CL_INT_IB;
+        final int CL_INT_IX =  Bindingtest1.CL_INT_IX;
+        iexp = 10;
+        Assert.assertEquals(iexp++, CL_INT_I0);
+        Assert.assertEquals(iexp++, CL_INT_I1);
+        Assert.assertEquals(iexp++, CL_INT_I2);
+        Assert.assertEquals(iexp++, CL_INT_I3);
+        Assert.assertEquals(iexp++, CL_INT_I4);
+        Assert.assertEquals(iexp++, CL_INT_I5);
+        Assert.assertEquals(iexp++, CL_INT_I6);
+        Assert.assertEquals(iexp++, CL_INT_I7);
+        Assert.assertEquals(iexp++, CL_INT_I8);
+        Assert.assertEquals(iexp++, CL_INT_I9);
+        Assert.assertEquals(iexp++, CL_INT_IA);
+        Assert.assertEquals(iexp++, CL_INT_IB);
+        Assert.assertEquals(0xffffffff, CL_INT_IX);
+
+        // Integer 64bit (long / define )
+        final long CL_LNG_L0 =  Bindingtest1.CL_LNG_L0;
+        final long CL_LNG_L1 =  Bindingtest1.CL_LNG_L1;
+        final long CL_LNG_L2 =  Bindingtest1.CL_LNG_L2;
+        final long CL_LNG_L3 =  Bindingtest1.CL_LNG_L3;
+        final long CL_LNG_L4 = -Bindingtest1.CL_LNG_L4;
+        final long CL_LNG_L5 = -Bindingtest1.CL_LNG_L5;
+        final long CL_LNG_L6 = -Bindingtest1.CL_LNG_L6;
+        final long CL_LNG_L7 = -Bindingtest1.CL_LNG_L7;
+        final long CL_LNG_L8 =  Bindingtest1.CL_LNG_L8;
+        final long CL_LNG_L9 =  Bindingtest1.CL_LNG_L9;
+        final long CL_LNG_LA =  Bindingtest1.CL_LNG_LA;
+        final long CL_LNG_LB =  Bindingtest1.CL_LNG_LB;
+        final long CL_LNG_LX =  Bindingtest1.CL_LNG_LX;
+        long lexp = 2147483648L;
+        Assert.assertEquals(lexp++, CL_LNG_L0);
+        Assert.assertEquals(lexp++, CL_LNG_L1);
+        Assert.assertEquals(lexp++, CL_LNG_L2);
+        Assert.assertEquals(lexp++, CL_LNG_L3);
+        Assert.assertEquals(lexp++, CL_LNG_L4);
+        Assert.assertEquals(lexp++, CL_LNG_L5);
+        Assert.assertEquals(lexp++, CL_LNG_L6);
+        Assert.assertEquals(lexp++, CL_LNG_L7);
+        Assert.assertEquals(lexp++, CL_LNG_L8);
+        Assert.assertEquals(lexp++, CL_LNG_L9);
+        Assert.assertEquals(lexp++, CL_LNG_LA);
+        Assert.assertEquals(lexp++, CL_LNG_LB);
+        Assert.assertEquals(0xffffffffffffffffL, CL_LNG_LX);
+
+        // Floating point hexadecimals
+        final float CL_FLT_A0 = Bindingtest1.CL_FLT_A0;
+        final float CL_FLT_A1 = Bindingtest1.CL_FLT_A1;
+        final float CL_FLT_A2 = Bindingtest1.CL_FLT_A2;
+        final float CL_FLT_A3 = Bindingtest1.CL_FLT_A3;
+        final float CL_FLT_A4 = Bindingtest1.CL_FLT_A4;
+        final float CL_FLT_A5 = Bindingtest1.CL_FLT_A5;
+        final float CL_FLT_A6 = Bindingtest1.CL_FLT_A6;
+        final float CL_FLT_A7 = Bindingtest1.CL_FLT_A7;
+        Assert.assertEquals(  0x1.p127f,  CL_FLT_A0, EPSILON);
+        Assert.assertEquals(  0x1.p+127F, CL_FLT_A1, EPSILON);
+        Assert.assertEquals(  0x1.p-127f, CL_FLT_A2, EPSILON);
+        Assert.assertEquals( -0.1f, CL_FLT_A3, EPSILON);
+        Assert.assertEquals(  0.2f, CL_FLT_A4, EPSILON);
+        Assert.assertEquals(  0.3f, CL_FLT_A5, EPSILON);
+        Assert.assertEquals(  0.4f, CL_FLT_A6, EPSILON);
+        Assert.assertEquals(  1.0f, CL_FLT_A7, EPSILON);
+
+        final float CL_FLT_EPSILON = Bindingtest1.CL_FLT_EPSILON;
+        final double CL_FLT_MAX= Bindingtest1.CL_FLT_MAX;
+        final double CL_FLT_MIN = Bindingtest1.CL_FLT_MIN;
+        Assert.assertEquals(  0x1.0p-23f, CL_FLT_EPSILON, EPSILON);
+        Assert.assertEquals(  0x1.fffffep127f, CL_FLT_MAX, EPSILON);
+        Assert.assertEquals(  0x1.0p-126f, CL_FLT_MIN, EPSILON);
+
+        final double CL_DBL_B0 = Bindingtest1.CL_DBL_B0;
+        final double CL_DBL_B1 = Bindingtest1.CL_DBL_B1;
+        final double CL_DBL_B2 = Bindingtest1.CL_DBL_B2;
+        final double CL_DBL_B3 = Bindingtest1.CL_DBL_B3;
+        final double CL_DBL_B4 = Bindingtest1.CL_DBL_B4;
+        final double CL_DBL_B5 = Bindingtest1.CL_DBL_B5;
+        final double CL_DBL_B6 = Bindingtest1.CL_DBL_B6;
+        Assert.assertEquals(  0x1.p127d,  CL_DBL_B0, EPSILON);
+        Assert.assertEquals(  0x1.p+127D, CL_DBL_B1, EPSILON);
+        Assert.assertEquals(  0x1.p-127d, CL_DBL_B2, EPSILON);
+        Assert.assertEquals( -0.1, CL_DBL_B3, EPSILON);
+        Assert.assertEquals(  0.2, CL_DBL_B4, EPSILON);
+        Assert.assertEquals(  0.3, CL_DBL_B5, EPSILON);
+        Assert.assertEquals(  3.5e+38, CL_DBL_B6, EPSILON);
+
+        final float CL_DBL_EPSILON = Bindingtest1.CL_DBL_EPSILON;
+        final double CL_DBL_MAX= Bindingtest1.CL_DBL_MAX;
+        final double CL_DBL_MIN = Bindingtest1.CL_DBL_MIN;
+        Assert.assertEquals(  0x1.0p-52f, CL_DBL_EPSILON, EPSILON);
+        Assert.assertEquals(  0x1.fffffffffffffp1023, CL_DBL_MAX, EPSILON);
+        Assert.assertEquals(  0x1.0p-1022, CL_DBL_MIN, EPSILON);
+    }
+
     ByteBuffer newByteBuffer(final int size, final boolean direct) {
         if(direct) {
             final ByteBuffer bb = Buffers.newDirectByteBuffer(size);
@@ -629,6 +841,66 @@ public class BaseClass extends JunitTracer {
                   Assert.assertTrue("Wrong result: 0x"+Long.toHexString(pb.get(i))+"+1 != 0x"+Long.toHexString(pb2.get(i)), (pb.get(i)+1)==pb2.get(i));
               }
           }
+
+          {
+              final long l0 = 0xAAFFEE;
+              final long l1 = binding.testXID(l0);
+              final long l2 = binding.testXID_2(l0);
+              Assert.assertEquals(l0, l1);
+              Assert.assertEquals(l0, l2);
+
+              final ByteBuffer bb = Buffers.newDirectByteBuffer(PointerBuffer.ELEMENT_SIZE);
+              for(int j=0; j<bb.limit(); j++) {
+                  bb.put(j, (byte)(0xAA+j));
+              }
+              final ByteBuffer bbOut = binding.testAnonBuffer(bb);
+              Assert.assertEquals(bb, bbOut);
+
+              final ShortBlob sb = ShortBlob.create();
+              sb.setB1((byte)0xAA);
+              sb.setB2((byte)0xEE);
+              final ShortBlob sb_ = binding.testShortBlob(sb);
+              final ShortBlob sb0 = binding.testLPShortBlob0(sb);
+              final ShortBlob sb1 = binding.testLPShortBlob1(sb);
+              final ShortBlob sb2 = binding.testLPShortBlob2(sb);
+              final ShortBlob sb3 = binding.testLPShortBlob3(sb);
+              final ShortBlob sb4 = binding.testShortBlobL1(sb);
+              final ShortBlob sb5 = binding.testShortBlobL2(sb);
+              Assert.assertEquals(sb.getBuffer(), sb_.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb0.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb1.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb2.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb3.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb4.getBuffer());
+              Assert.assertEquals(sb.getBuffer(), sb5.getBuffer());
+
+              final Int32Struct i32s = Int32Struct.create();
+              i32s.setB1((byte)0x02);
+              i32s.setB2((byte)0x12);
+              i32s.setB3((byte)0x22);
+              i32s.setB4((byte)0x32);
+              final Int32Struct i32s0 = binding.testInt32Struct(i32s);
+              Assert.assertEquals(i32s.getBuffer(), i32s0.getBuffer());
+
+              final AnonBlob ab = binding.testCreateAnonBlob();
+              binding.testDestroyAnonBlob(ab);
+
+              final long ab2 = binding.testCreateAnonBlob2();
+              binding.testDestroyAnonBlob2(ab2);
+
+              final long[] foo = new long[] { 0x1122334455667788L };
+              final LongBuffer fooLB = Buffers.newDirectLongBuffer(foo);
+              final LongBuffer foo1Out = binding.testFooPtr(fooLB);
+              Assert.assertEquals(fooLB, foo1Out);
+              final LongBuffer foo2Out = binding.testFooPtr(foo, 0);
+              Assert.assertEquals(fooLB, foo2Out);
+          }
+
+          {
+              i=41;
+              final int iRes = binding.testDelegate(i);
+              Assert.assertEquals(i+1, iRes);
+          }
     }
 
     public void chapter04TestPointerBuffer(final Bindingtest1 binding) throws Exception {
@@ -824,8 +1096,8 @@ public class BaseClass extends JunitTracer {
 
     public void chapter09TestCompoundAlignment(final Bindingtest1 binding) throws Exception {
 
-        final MachineDescription.StaticConfig smd = MachineDescriptionRuntime.getStatic();
-        final MachineDescription md = MachineDescriptionRuntime.getRuntime();
+        final MachineDataInfo.StaticConfig smd = MachineDataInfoRuntime.getStatic();
+        final MachineDataInfo md = MachineDataInfoRuntime.getRuntime();
 
         System.err.println("static  md: "+smd);
         System.err.println("runtime md: "+md);
@@ -1115,22 +1387,20 @@ public class BaseClass extends JunitTracer {
         }
     }
 
-    public static final float EPSILON = 1.1920929E-7f; // Float.MIN_VALUE == 1.4e-45f ; double EPSILON 2.220446049250313E-16d
-
     /** Test array and pointer bindings of structs  */
     public void chapter12TestStructArrayModelConst(final Bindingtest1 binding) throws Exception {
         final TK_ModelConst model = binding.createModelConst();
 
         Assert.assertEquals(3, model.getIntxxPointerCustomLenVal());
         Assert.assertEquals(3, model.getInt32PointerCustomLenVal());
-        Assert.assertEquals(3, model.getInt32ArrayFixedLenArrayLength());
-        Assert.assertEquals(3, model.getStructArrayFixedLenArrayLength());
+        Assert.assertEquals(3, TK_ModelConst.getInt32ArrayFixedLenArrayLength());
+        Assert.assertEquals(3, TK_ModelConst.getStructArrayFixedLenArrayLength());
         Assert.assertEquals(3, model.getStructPointerCustomLenVal());
 
         // field: int32ArrayFixedLen
         //        CType['int32_t *', size [fixed false, lnx64 12], [array*1]], with array length of 3
         {
-            final int size = model.getInt32ArrayFixedLenArrayLength();
+            final int size = TK_ModelConst.getInt32ArrayFixedLenArrayLength();
             final int[] all = model.getInt32ArrayFixedLen(0, new int[size]);
             final IntBuffer allB = model.getInt32ArrayFixedLen();
             Assert.assertEquals(size, allB.limit());
@@ -1170,7 +1440,7 @@ public class BaseClass extends JunitTracer {
         // field: mat4x4
         //        CType['float * *', size [fixed false, lnx64 64], [array*2]], with array length of <code>4*4</code> */
         {
-            Assert.assertEquals(4*4, model.getMat4x4ArrayLength());
+            Assert.assertEquals(4*4, TK_ModelConst.getMat4x4ArrayLength());
             final FloatBuffer mat4x4 = model.getMat4x4();
             Assert.assertEquals(4*4, mat4x4.limit());
             for(int i=0; i<4; i++) {
@@ -1185,7 +1455,7 @@ public class BaseClass extends JunitTracer {
         // field: structArrayFixedLen
         //        field: CType['TK_Dimension *', size [fixed false, lnx64 48], [array*1]], with array length of 3
         {
-            final int size = model.getStructArrayFixedLenArrayLength();
+            final int size = TK_ModelConst.getStructArrayFixedLenArrayLength();
             final TK_Dimension[] all = model.getStructArrayFixedLen(0, new TK_Dimension[size]);
             for(int i=0; i<size; i++) {
                 Assert.assertEquals(51 + i * 10, all[i].getX());
@@ -1236,7 +1506,7 @@ public class BaseClass extends JunitTracer {
         assertAPTR(surfaceContext, model.getCtx());
 
         {
-            Assert.assertEquals(12, model.getModelNameArrayFixedLenArrayLength());
+            Assert.assertEquals(12, TK_ModelConst.getModelNameArrayFixedLenArrayLength());
 
             final ByteBuffer bb = model.getModelNameArrayFixedLen();
             Assert.assertEquals(12, bb.limit());
@@ -1296,14 +1566,14 @@ public class BaseClass extends JunitTracer {
 
         Assert.assertEquals(3, model.getIntxxPointerCustomLenVal());
         Assert.assertEquals(3, model.getInt32PointerCustomLenVal());
-        Assert.assertEquals(3, model.getInt32ArrayFixedLenArrayLength());
-        Assert.assertEquals(3, model.getStructArrayFixedLenArrayLength());
+        Assert.assertEquals(3, TK_ModelMutable.getInt32ArrayFixedLenArrayLength());
+        Assert.assertEquals(3, TK_ModelMutable.getStructArrayFixedLenArrayLength());
         Assert.assertEquals(3, model.getStructPointerCustomLenVal());
 
         // field: int32ArrayFixedLen
         //        CType['int32_t *', size [fixed false, lnx64 12], [array*1]], with array length of 3
         {
-            final int size = model.getInt32ArrayFixedLenArrayLength();
+            final int size = TK_ModelMutable.getInt32ArrayFixedLenArrayLength();
             {
                 final int[] values = new int[] { 1, 2, 3 };
                 model.setInt32ArrayFixedLen(0, values);
@@ -1385,7 +1655,7 @@ public class BaseClass extends JunitTracer {
             model.setMat4x4(2*4, new float[] { 31, 32, 33, 34 } );
             model.setMat4x4(3*4, new float[] { 41, 42, 43, 44 } );
 
-            Assert.assertEquals(4*4, model.getMat4x4ArrayLength());
+            Assert.assertEquals(4*4, TK_ModelMutable.getMat4x4ArrayLength());
             final FloatBuffer mat4x4 = model.getMat4x4();
             Assert.assertEquals(4*4, mat4x4.limit());
             for(int i=0; i<4; i++) {
@@ -1400,7 +1670,7 @@ public class BaseClass extends JunitTracer {
         // field: structArrayFixedLen
         //        field: CType['TK_Dimension *', size [fixed false, lnx64 48], [array*1]], with array length of 3
         {
-            final int size = model.getStructArrayFixedLenArrayLength();
+            final int size = TK_ModelMutable.getStructArrayFixedLenArrayLength();
             {
                 for(int i=0; i<size; i++) {
                     final TK_Dimension d = TK_Dimension.create();
