@@ -8,10 +8,67 @@
 
 #define DEBUG 1
 
+MYAPI XID MYAPIENTRY testXID(XID v) {
+    return v;
+}
+MYAPI XID_2 MYAPIENTRY testXID_2(XID_2 v) {
+    return v;
+}
+MYAPI AnonBuffer MYAPIENTRY testAnonBuffer(AnonBuffer v) {
+    return v;
+}
+
+MYAPI const ShortBlob * MYAPIENTRY testShortBlob(const ShortBlob *v) {
+    return v;
+}
+MYAPI const LPShortBlob0 MYAPIENTRY testLPShortBlob0(const LPShortBlob0 v) {
+    return v;
+}
+MYAPI LPShortBlob1 MYAPIENTRY testLPShortBlob1(LPShortBlob1 v) {
+    return v;
+}
+MYAPI const LPShortBlob2 MYAPIENTRY testLPShortBlob2(const LPShortBlob2 v) {
+    return v;
+}
+MYAPI LPShortBlob3 MYAPIENTRY testLPShortBlob3(LPShortBlob3 v) {
+    return v;
+}
+MYAPI const ShortBlobL1 * MYAPIENTRY testShortBlobL1(const ShortBlobL1 * v) {
+    return v;
+}
+MYAPI ShortBlobL2 * MYAPIENTRY testShortBlobL2(ShortBlobL2 * v) {
+    return v;
+}
+MYAPI struct Int32Struct * MYAPIENTRY testInt32Struct(struct Int32Struct * v) {
+    return v;
+}
+
+MYAPI AnonBlob MYAPIENTRY testCreateAnonBlob() {
+    return (AnonBlob) calloc(1, sizeof(char));
+}
+MYAPI void MYAPIENTRY testDestroyAnonBlob(AnonBlob v) {
+    free(v);
+}
+
+MYAPI struct _AnonBlob2 * MYAPIENTRY testCreateAnonBlob2() {
+    return (struct _AnonBlob2 *) calloc(1, sizeof(char));
+}
+MYAPI void MYAPIENTRY testDestroyAnonBlob2(struct _AnonBlob2 * v) {
+    free(v);
+}
+
+MYAPI foo_ptr MYAPIENTRY testFooPtr(foo_ptr v) {
+    return v;
+}
+
 MYAPI foo MYAPIENTRY nopTest() {
     return 42;
 }
 
+MYAPI int32_t MYAPIENTRY testDelegate(int32_t v) {
+   return v; 
+}
+
 /**
  * new blob sizeof(void*) filled w/ 0xDEADBEEF
  */
